﻿/*    
Nemulation: A .NET based framework/platform for virtualization applications
Copyright (C) 2011 Bryan Perris

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using NEmu;
using NEmu.Services;

namespace NEmu.Services
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single, ConcurrencyMode=ConcurrencyMode.Multiple)]
    [ServiceKnownType(typeof(RunState))]
    public sealed class VirtualHostService : IVirtualHostService
    {
        VMHostContainer m_Container = new VMHostContainer();

        public VMHostContainer VMContainer
        {
            get { return m_Container; }
        }

        #region IVirtualHostService Members

        public VirtualMachineStatus[] GetMachineList()
        {
            VirtualMachineStatus[] statuses = new VirtualMachineStatus[m_Container.Count];
            int i = 0;

            foreach (VirtualMachine machine in m_Container)
            {
                VirtualMachineStatus stat = new VirtualMachineStatus();
                stat.MachineID = machine.GetHashCode();
                stat.MachineType = machine.GetType().ToString();
                stat.RunningState = machine.CurrentRunState;
                statuses[i++] = stat;
            }

            return statuses;
        }

        public void SetMachineState(int hash, RunState state)
        {
            m_Container.SetState(hash, state);
        }

        public void SetAllMachinesState(RunState state)
        {
            m_Container.SetState(state);
        }

        #endregion
    }
}
