﻿/*    
Nemulation: A .NET based framework/platform for virtualization applications
Copyright (C) 2011 Bryan Perris

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NEmu.Debugging;
using System.Globalization;

namespace NEmu.Code.Emitting
{
    public sealed class Recompiler<TEmitterBase> where TEmitterBase : EmitterBase
    {
        object m_CPUCore;
        Dictionary<long, CompiledFunction> m_CompilerTable;
        int m_CodeCacheSize = 30;
        EmitterBase m_Emitter;

        public Recompiler(object cpuCore)
        {
            m_CPUCore = cpuCore;
            m_CompilerTable = new Dictionary<long, CompiledFunction>();
            m_Emitter = Activator.CreateInstance<TEmitterBase>();
        }

        public void ClearCodeCache()
        {
            m_CompilerTable.Clear();
        }

        public void CompileBlock(long address)
        {
            if (m_CodeCacheSize > 30)
                ClearCodeCache();

            m_CompilerTable.Add(address, m_Emitter.CompileFunction(address, m_CPUCore));

            EmuLog.DebugLog("Compiled function_" + address.ToString("X", CultureInfo.CurrentCulture));
        }

        public void ExecuteBlock(long address)
        {
            CompiledFunction funct;
            if (m_CompilerTable.TryGetValue(address, out funct))
                funct(m_CPUCore);
        }

        /// <summary>
        /// The number of delegates the recompiler can store
        /// </summary>
        public int CodeCacheSize
        {
            get { return m_CodeCacheSize; }
            set { m_CodeCacheSize = value; }
        }
    }
}
