﻿/*    
Nemulation: A .NET based framework/platform for virtualization applications
Copyright (C) 2011 Bryan Perris

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NEmu.Memory.MemoryPages
{
    [Serializable]
    public sealed class BufferPage : MemoryPage
    {
        private byte[] m_Memory;

        public BufferPage(int size)
        {
            m_Memory = new byte[size];
        }

        public override int Read(int address)
        {
            #if DEBUG
                if (address >= m_Memory.Length) return -1;
             #endif

            return m_Memory[address];
        }

        public override void Write(int address, byte value)
        {
            #if DEBUG
                if (address >= m_Memory.Length) return;
            #endif

            m_Memory[address] = value;
        }

        public override int Size
        {
            get { return m_Memory.Length; }
        }

        public override void Clear()
        {
            Array.Clear(m_Memory, 0, m_Memory.Length);
        }
    }
}
