﻿/*    
Nemulation: A .NET based framework/platform for virtualization applications
Copyright (C) 2011 Bryan Perris

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;

namespace NEmuVirtualHost.Services
{
    public sealed class ServicesProviderEnumerator : IEnumerator<KeyValuePair<Type, ServiceHost>>
    {
        private ServicesProvider m_Provider;
        private KeyValuePair<Type, ServiceHost> m_CurrentKeyPair = new KeyValuePair<Type,ServiceHost>();
        private IEnumerator<Type> m_KeyEnumerator;

        internal ServicesProviderEnumerator(ServicesProvider provider)
        {
            m_Provider = provider;
            m_KeyEnumerator = provider.Keys.GetEnumerator();
        }

        KeyValuePair<Type, ServiceHost> IEnumerator<KeyValuePair<Type, ServiceHost>>.Current
        {
            get { return m_CurrentKeyPair; }
        }

        void IDisposable.Dispose()
        {
            m_Provider = null;
            m_KeyEnumerator.Dispose();
        }

        object System.Collections.IEnumerator.Current
        {
            get { return this.m_CurrentKeyPair.Value; }
        }

        bool System.Collections.IEnumerator.MoveNext()
        {
            bool result = m_KeyEnumerator.MoveNext();

            if (result)
            {
                m_CurrentKeyPair = new KeyValuePair<Type, ServiceHost>(m_KeyEnumerator.Current, m_Provider[m_KeyEnumerator.Current]);
                return true;
            }
            else
            {
                return false;
            }
        }

        void System.Collections.IEnumerator.Reset()
        {
            m_CurrentKeyPair = new KeyValuePair<Type, ServiceHost>();
        }
    }
}
